@@ -12,21 +12,23 @@
 use crate::core::resolver::context::Context;
 use crate::core::resolver::errors::describe_path;
 use crate::core::resolver::types::{ConflictReason, DepInfo, FeaturesSet};
-use crate::core::resolver::{ActivateResult, ResolveOpts};
+use crate::core::resolver::{
+    ActivateError, ActivateResult, CliFeatures, RequestedFeatures, ResolveOpts, VersionOrdering,
+    VersionPreferences,
+};
 use crate::core::{Dependency, FeatureValue, PackageId, PackageIdSpec, Registry, Summary};
-use crate::core::{GitReference, SourceId};
-use crate::util::errors::{CargoResult, CargoResultExt};
+use crate::util::errors::CargoResult;
 use crate::util::interning::InternedString;
-use crate::util::Config;
+
+use anyhow::Context as _;
 use log::debug;
-use std::cmp::Ordering;
 use std::collections::{BTreeSet, HashMap, HashSet};
 use std::rc::Rc;
 
 pub struct RegistryQueryer<'a> {
     pub registry: &'a mut (dyn Registry + 'a),
     replacements: &'a [(PackageIdSpec, Dependency)],
-    try_to_use: &'a HashSet<PackageId>,
+    version_prefs: &'a VersionPreferences,
     /// If set the list of dependency candidates will be sorted by minimal
     /// versions first. That allows `cargo update -Z minimal-versions` which will
     /// specify minimum dependency versions to be used.
@@ -40,30 +42,23 @@ pub struct RegistryQueryer<'a> {
     >,
     /// all the cases we ended up using a supplied replacement
     used_replacements: HashMap<PackageId, Summary>,
-    /// Where to print warnings, if configured.
-    config: Option<&'a Config>,
-    /// Sources that we've already wared about possibly colliding in the future.
-    warned_git_collisions: HashSet<SourceId>,
 }
 
 impl<'a> RegistryQueryer<'a> {
     pub fn new(
         registry: &'a mut dyn Registry,
         replacements: &'a [(PackageIdSpec, Dependency)],
-        try_to_use: &'a HashSet<PackageId>,
+        version_prefs: &'a VersionPreferences,
         minimal_versions: bool,
-        config: Option<&'a Config>,
     ) -> Self {
         RegistryQueryer {
             registry,
             replacements,
-            try_to_use,
+            version_prefs,
             minimal_versions,
             registry_cache: HashMap::new(),
             summary_cache: HashMap::new(),
             used_replacements: HashMap::new(),
-            config,
-            warned_git_collisions: HashSet::new(),
         }
     }
 
@@ -75,44 +70,6 @@ impl<'a> RegistryQueryer<'a> {
         self.used_replacements.get(&p)
     }
 
-    /// Issues a future-compatible warning targeted at removing reliance on
-    /// unifying behavior between these two dependency directives:
-    ///
-    /// ```toml
-    /// [dependencies]
-    /// a = { git = 'https://example.org/foo' }
-    /// a = { git = 'https://example.org/foo', branch = 'master }
-    /// ```
-    ///
-    /// Historical versions of Cargo considered these equivalent but going
-    /// forward we'd like to fix this. For more details see the comments in
-    /// src/cargo/sources/git/utils.rs
-    fn warn_colliding_git_sources(&mut self, id: SourceId) -> CargoResult<()> {
-        let config = match self.config {
-            Some(config) => config,
-            None => return Ok(()),
-        };
-        let prev = match self.warned_git_collisions.replace(id) {
-            Some(prev) => prev,
-            None => return Ok(()),
-        };
-        match (id.git_reference(), prev.git_reference()) {
-            (Some(GitReference::DefaultBranch), Some(GitReference::Branch(b)))
-            | (Some(GitReference::Branch(b)), Some(GitReference::DefaultBranch))
-                if b == "master" => {}
-            _ => return Ok(()),
-        }
-
-        config.shell().warn(&format!(
-            "two git dependencies found for `{}` \
-             where one uses `branch = \"master\"` and the other doesn't; \
-             this will break in a future version of Cargo, so please \
-             ensure the dependency forms are consistent",
-            id.url(),
-        ))?;
-        Ok(())
-    }
-
     /// Queries the `registry` to return a list of candidates for `dep`.
     ///
     /// This method is the location where overrides are taken into account. If
@@ -120,7 +77,6 @@ impl<'a> RegistryQueryer<'a> {
     /// applied by performing a second query for what the override should
     /// return.
     pub fn query(&mut self, dep: &Dependency) -> CargoResult<Rc<Vec<Summary>>> {
-        self.warn_colliding_git_sources(dep.source_id())?;
         if let Some(out) = self.registry_cache.get(dep).cloned() {
             return Ok(out);
         }
@@ -208,28 +164,16 @@ impl<'a> RegistryQueryer<'a> {
             }
         }
 
-        // When we attempt versions for a package we'll want to do so in a
-        // sorted fashion to pick the "best candidates" first. Currently we try
-        // prioritized summaries (those in `try_to_use`) and failing that we
-        // list everything from the maximum version to the lowest version.
-        ret.sort_unstable_by(|a, b| {
-            let a_in_previous = self.try_to_use.contains(&a.package_id());
-            let b_in_previous = self.try_to_use.contains(&b.package_id());
-            let previous_cmp = a_in_previous.cmp(&b_in_previous).reverse();
-            match previous_cmp {
-                Ordering::Equal => {
-                    let cmp = a.version().cmp(b.version());
-                    if self.minimal_versions {
-                        // Lower version ordered first.
-                        cmp
-                    } else {
-                        // Higher version ordered first.
-                        cmp.reverse()
-                    }
-                }
-                _ => previous_cmp,
-            }
-        });
+        // When we attempt versions for a package we'll want to do so in a sorted fashion to pick
+        // the "best candidates" first. VersionPreferences implements this notion.
+        self.version_prefs.sort_summaries(
+            &mut ret,
+            if self.minimal_versions {
+                VersionOrdering::MinimumVersionsFirst
+            } else {
+                VersionOrdering::MaximumVersionsFirst
+            },
+        );
 
         let out = Rc::new(ret);
 
@@ -268,8 +212,8 @@ impl<'a> RegistryQueryer<'a> {
         let mut deps = deps
             .into_iter()
             .map(|(dep, features)| {
-                let candidates = self.query(&dep).chain_err(|| {
-                    anyhow::format_err!(
+                let candidates = self.query(&dep).with_context(|| {
+                    format!(
                         "failed to get `{}` as a dependency of {}",
                         dep.package_name(),
                         describe_path(&cx.parents.path_to_bottom(&candidate.package_id())),
@@ -307,10 +251,10 @@ pub fn resolve_features<'b>(
     let deps = s.dependencies();
     let deps = deps.iter().filter(|d| d.is_transitive() || opts.dev_deps);
 
-    let reqs = build_requirements(s, opts)?;
+    let reqs = build_requirements(parent, s, opts)?;
     let mut ret = Vec::new();
-    let mut used_features = HashSet::new();
-    let default_dep = (false, BTreeSet::new());
+    let default_dep = BTreeSet::new();
+    let mut valid_dep_names = HashSet::new();
 
     // Next, collect all actually enabled dependencies and their features.
     for dep in deps {
@@ -319,114 +263,117 @@ pub fn resolve_features<'b>(
         if dep.is_optional() && !reqs.deps.contains_key(&dep.name_in_toml()) {
             continue;
         }
+        valid_dep_names.insert(dep.name_in_toml());
         // So we want this dependency. Move the features we want from
         // `feature_deps` to `ret` and register ourselves as using this
         // name.
-        let base = reqs.deps.get(&dep.name_in_toml()).unwrap_or(&default_dep);
-        used_features.insert(dep.name_in_toml());
-        let always_required = !dep.is_optional()
-            && !s
-                .dependencies()
-                .iter()
-                .any(|d| d.is_optional() && d.name_in_toml() == dep.name_in_toml());
-        if always_required && base.0 {
-            return Err(match parent {
-                None => anyhow::format_err!(
-                    "Package `{}` does not have feature `{}`. It has a required dependency \
-                     with that name, but only optional dependencies can be used as features.",
-                    s.package_id(),
-                    dep.name_in_toml()
-                )
-                .into(),
-                Some(p) => (
-                    p,
-                    ConflictReason::RequiredDependencyAsFeatures(dep.name_in_toml()),
-                )
-                    .into(),
-            });
-        }
-        let mut base = base.1.clone();
+        let mut base = reqs
+            .deps
+            .get(&dep.name_in_toml())
+            .unwrap_or(&default_dep)
+            .clone();
         base.extend(dep.features().iter());
-        for feature in base.iter() {
-            if feature.contains('/') {
-                return Err(anyhow::format_err!(
-                    "feature names may not contain slashes: `{}`",
-                    feature
-                )
-                .into());
-            }
-        }
         ret.push((dep.clone(), Rc::new(base)));
     }
 
-    // Any entries in `reqs.dep` which weren't used are bugs in that the
-    // package does not actually have those dependencies. We classified
-    // them as dependencies in the first place because there is no such
-    // feature, either.
-    let remaining = reqs
-        .deps
-        .keys()
-        .cloned()
-        .filter(|s| !used_features.contains(s))
-        .collect::<Vec<_>>();
-    if !remaining.is_empty() {
-        let features = remaining.join(", ");
-        return Err(match parent {
-            None => anyhow::format_err!(
-                "Package `{}` does not have these features: `{}`",
-                s.package_id(),
-                features
-            )
-            .into(),
-            Some(p) => (p, ConflictReason::MissingFeatures(features)).into(),
-        });
+    // This is a special case for command-line `--features
+    // dep_name/feat_name` where `dep_name` does not exist. All other
+    // validation is done either in `build_requirements` or
+    // `build_feature_map`.
+    for dep_name in reqs.deps.keys() {
+        if !valid_dep_names.contains(dep_name) {
+            let e = RequirementError::MissingDependency(*dep_name);
+            return Err(e.into_activate_error(parent, s));
+        }
     }
 
-    Ok((reqs.into_used(), ret))
+    Ok((reqs.into_features(), ret))
 }
 
 /// Takes requested features for a single package from the input `ResolveOpts` and
 /// recurses to find all requested features, dependencies and requested
 /// dependency features in a `Requirements` object, returning it to the resolver.
 fn build_requirements<'a, 'b: 'a>(
+    parent: Option<PackageId>,
     s: &'a Summary,
     opts: &'b ResolveOpts,
-) -> CargoResult<Requirements<'a>> {
+) -> ActivateResult<Requirements<'a>> {
     let mut reqs = Requirements::new(s);
 
-    if opts.features.all_features {
-        for key in s.features().keys() {
-            reqs.require_feature(*key)?;
+    let handle_default = |uses_default_features, reqs: &mut Requirements<'_>| {
+        if uses_default_features && s.features().contains_key("default") {
+            if let Err(e) = reqs.require_feature(InternedString::new("default")) {
+                return Err(e.into_activate_error(parent, s));
+            }
         }
-        for dep in s.dependencies().iter().filter(|d| d.is_optional()) {
-            reqs.require_dependency(dep.name_in_toml());
-        }
-    } else {
-        for &f in opts.features.features.iter() {
-            reqs.require_value(&FeatureValue::new(f, s))?;
-        }
-    }
+        Ok(())
+    };
 
-    if opts.features.uses_default_features && s.features().contains_key("default") {
-        reqs.require_feature(InternedString::new("default"))?;
+    match &opts.features {
+        RequestedFeatures::CliFeatures(CliFeatures {
+            features,
+            all_features,
+            uses_default_features,
+        }) => {
+            if *all_features {
+                for key in s.features().keys() {
+                    if let Err(e) = reqs.require_feature(*key) {
+                        return Err(e.into_activate_error(parent, s));
+                    }
+                }
+            } else {
+                for fv in features.iter() {
+                    if let Err(e) = reqs.require_value(fv) {
+                        return Err(e.into_activate_error(parent, s));
+                    }
+                }
+                handle_default(*uses_default_features, &mut reqs)?;
+            }
+        }
+        RequestedFeatures::DepFeatures {
+            features,
+            uses_default_features,
+        } => {
+            for feature in features.iter() {
+                if let Err(e) = reqs.require_feature(*feature) {
+                    return Err(e.into_activate_error(parent, s));
+                }
+            }
+            handle_default(*uses_default_features, &mut reqs)?;
+        }
     }
 
     Ok(reqs)
 }
 
+/// Set of feature and dependency requirements for a package.
+#[derive(Debug)]
 struct Requirements<'a> {
     summary: &'a Summary,
-    // The deps map is a mapping of package name to list of features enabled.
-    // Each package should be enabled, and each package should have the
-    // specified set of features enabled. The boolean indicates whether this
-    // package was specifically requested (rather than just requesting features
-    // *within* this package).
-    deps: HashMap<InternedString, (bool, BTreeSet<InternedString>)>,
-    // The used features set is the set of features which this local package had
-    // enabled, which is later used when compiling to instruct the code what
-    // features were enabled.
-    used: HashSet<InternedString>,
-    visited: HashSet<InternedString>,
+    /// The deps map is a mapping of dependency name to list of features enabled.
+    ///
+    /// The resolver will activate all of these dependencies, with the given
+    /// features enabled.
+    deps: HashMap<InternedString, BTreeSet<InternedString>>,
+    /// The set of features enabled on this package which is later used when
+    /// compiling to instruct the code what features were enabled.
+    features: HashSet<InternedString>,
+}
+
+/// An error for a requirement.
+///
+/// This will later be converted to an `ActivateError` depending on whether or
+/// not this is a dependency or a root package.
+enum RequirementError {
+    /// The package does not have the requested feature.
+    MissingFeature(InternedString),
+    /// The package does not have the requested dependency.
+    MissingDependency(InternedString),
+    /// A feature has a direct cycle to itself.
+    ///
+    /// Note that cycles through multiple features are allowed (but perhaps
+    /// they shouldn't be?).
+    Cycle(InternedString),
 }
 
 impl Requirements<'_> {
@@ -434,80 +381,149 @@ impl Requirements<'_> {
         Requirements {
             summary,
             deps: HashMap::new(),
-            used: HashSet::new(),
-            visited: HashSet::new(),
+            features: HashSet::new(),
         }
     }
 
-    fn into_used(self) -> HashSet<InternedString> {
-        self.used
+    fn into_features(self) -> HashSet<InternedString> {
+        self.features
     }
 
-    fn require_crate_feature(&mut self, package: InternedString, feat: InternedString) {
+    fn require_dep_feature(
+        &mut self,
+        package: InternedString,
+        feat: InternedString,
+        weak: bool,
+    ) -> Result<(), RequirementError> {
         // If `package` is indeed an optional dependency then we activate the
         // feature named `package`, but otherwise if `package` is a required
         // dependency then there's no feature associated with it.
-        if self
-            .summary
-            .dependencies()
-            .iter()
-            .any(|dep| dep.name_in_toml() == package && dep.is_optional())
+        if !weak
+            && self
+                .summary
+                .dependencies()
+                .iter()
+                .any(|dep| dep.name_in_toml() == package && dep.is_optional())
         {
-            self.used.insert(package);
-        }
-        self.deps
-            .entry(package)
-            .or_insert((false, BTreeSet::new()))
-            .1
-            .insert(feat);
-    }
-
-    fn seen(&mut self, feat: InternedString) -> bool {
-        if self.visited.insert(feat) {
-            self.used.insert(feat);
-            false
-        } else {
-            true
+            self.require_feature(package)?;
         }
+        self.deps.entry(package).or_default().insert(feat);
+        Ok(())
     }
 
     fn require_dependency(&mut self, pkg: InternedString) {
-        if self.seen(pkg) {
-            return;
-        }
-        self.deps.entry(pkg).or_insert((false, BTreeSet::new())).0 = true;
+        self.deps.entry(pkg).or_default();
     }
 
-    fn require_feature(&mut self, feat: InternedString) -> CargoResult<()> {
-        if feat.is_empty() || self.seen(feat) {
+    fn require_feature(&mut self, feat: InternedString) -> Result<(), RequirementError> {
+        if !self.features.insert(feat) {
+            // Already seen this feature.
             return Ok(());
         }
-        for fv in self
-            .summary
-            .features()
-            .get(&feat)
-            .expect("must be a valid feature")
-        {
-            match *fv {
-                FeatureValue::Feature(ref dep_feat) if **dep_feat == *feat => anyhow::bail!(
-                    "cyclic feature dependency: feature `{}` depends on itself",
-                    feat
-                ),
-                _ => {}
+
+        let fvs = match self.summary.features().get(&feat) {
+            Some(fvs) => fvs,
+            None => return Err(RequirementError::MissingFeature(feat)),
+        };
+
+        for fv in fvs {
+            if let FeatureValue::Feature(dep_feat) = fv {
+                if *dep_feat == feat {
+                    return Err(RequirementError::Cycle(feat));
+                }
             }
             self.require_value(fv)?;
         }
         Ok(())
     }
 
-    fn require_value(&mut self, fv: &FeatureValue) -> CargoResult<()> {
+    fn require_value(&mut self, fv: &FeatureValue) -> Result<(), RequirementError> {
         match fv {
             FeatureValue::Feature(feat) => self.require_feature(*feat)?,
-            FeatureValue::Crate(dep) => self.require_dependency(*dep),
-            FeatureValue::CrateFeature(dep, dep_feat) => {
-                self.require_crate_feature(*dep, *dep_feat)
-            }
+            FeatureValue::Dep { dep_name } => self.require_dependency(*dep_name),
+            FeatureValue::DepFeature {
+                dep_name,
+                dep_feature,
+                // Weak features are always activated in the dependency
+                // resolver. They will be narrowed inside the new feature
+                // resolver.
+                weak,
+            } => self.require_dep_feature(*dep_name, *dep_feature, *weak)?,
         };
         Ok(())
     }
 }
+
+impl RequirementError {
+    fn into_activate_error(self, parent: Option<PackageId>, summary: &Summary) -> ActivateError {
+        match self {
+            RequirementError::MissingFeature(feat) => {
+                let deps: Vec<_> = summary
+                    .dependencies()
+                    .iter()
+                    .filter(|dep| dep.name_in_toml() == feat)
+                    .collect();
+                if deps.is_empty() {
+                    return match parent {
+                        None => ActivateError::Fatal(anyhow::format_err!(
+                            "Package `{}` does not have the feature `{}`",
+                            summary.package_id(),
+                            feat
+                        )),
+                        Some(p) => ActivateError::Conflict(
+                            p,
+                            ConflictReason::MissingFeatures(feat.to_string()),
+                        ),
+                    };
+                }
+                if deps.iter().any(|dep| dep.is_optional()) {
+                    match parent {
+                        None => ActivateError::Fatal(anyhow::format_err!(
+                            "Package `{}` does not have feature `{}`. It has an optional dependency \
+                             with that name, but that dependency uses the \"dep:\" \
+                             syntax in the features table, so it does not have an implicit feature with that name.",
+                            summary.package_id(),
+                            feat
+                        )),
+                        Some(p) => ActivateError::Conflict(
+                            p,
+                            ConflictReason::NonImplicitDependencyAsFeature(feat),
+                        ),
+                    }
+                } else {
+                    match parent {
+                        None => ActivateError::Fatal(anyhow::format_err!(
+                            "Package `{}` does not have feature `{}`. It has a required dependency \
+                             with that name, but only optional dependencies can be used as features.",
+                            summary.package_id(),
+                            feat
+                        )),
+                        Some(p) => ActivateError::Conflict(
+                            p,
+                            ConflictReason::RequiredDependencyAsFeature(feat),
+                        ),
+                    }
+                }
+            }
+            RequirementError::MissingDependency(dep_name) => {
+                match parent {
+                    None => ActivateError::Fatal(anyhow::format_err!(
+                        "package `{}` does not have a dependency named `{}`",
+                        summary.package_id(),
+                        dep_name
+                    )),
+                    // This code path currently isn't used, since `foo/bar`
+                    // and `dep:` syntax is not allowed in a dependency.
+                    Some(p) => ActivateError::Conflict(
+                        p,
+                        ConflictReason::MissingFeatures(dep_name.to_string()),
+                    ),
+                }
+            }
+            RequirementError::Cycle(feat) => ActivateError::Fatal(anyhow::format_err!(
+                "cyclic feature dependency: feature `{}` depends on itself",
+                feat
+            )),
+        }
+    }
+}
