@@ -1,7 +1,6 @@
 use std::time::SystemTime;
 
 use crate::*;
-use stacked_borrows::Tag;
 use thread::Time;
 
 // pthread_mutexattr_t is either 4 or 8 bytes, depending on the platform.
@@ -58,13 +57,15 @@ fn mutexattr_set_kind<'mir, 'tcx: 'mir>(
 // (the kind has to be at its offset for compatibility with static initializer macros)
 
 fn mutex_get_kind<'mir, 'tcx: 'mir>(
-    ecx: &mut MiriEvalContext<'mir, 'tcx>,
+    ecx: &MiriEvalContext<'mir, 'tcx>,
     mutex_op: &OpTy<'tcx, Tag>,
 ) -> InterpResult<'tcx, ScalarMaybeUninit<Tag>> {
     let offset = if ecx.pointer_size().bytes() == 8 { 16 } else { 12 };
     ecx.read_scalar_at_offset_atomic(
-        mutex_op, offset, ecx.machine.layouts.i32,
-        AtomicReadOp::Relaxed
+        mutex_op,
+        offset,
+        ecx.machine.layouts.i32,
+        AtomicReadOp::Relaxed,
     )
 }
 
@@ -75,8 +76,11 @@ fn mutex_set_kind<'mir, 'tcx: 'mir>(
 ) -> InterpResult<'tcx, ()> {
     let offset = if ecx.pointer_size().bytes() == 8 { 16 } else { 12 };
     ecx.write_scalar_at_offset_atomic(
-        mutex_op, offset, kind, ecx.machine.layouts.i32,
-        AtomicWriteOp::Relaxed
+        mutex_op,
+        offset,
+        kind,
+        ecx.machine.layouts.i32,
+        AtomicWriteOp::Relaxed,
     )
 }
 
@@ -84,10 +88,7 @@ fn mutex_get_id<'mir, 'tcx: 'mir>(
     ecx: &MiriEvalContext<'mir, 'tcx>,
     mutex_op: &OpTy<'tcx, Tag>,
 ) -> InterpResult<'tcx, ScalarMaybeUninit<Tag>> {
-    ecx.read_scalar_at_offset_atomic(
-        mutex_op, 4, ecx.machine.layouts.u32, 
-        AtomicReadOp::Relaxed
-    )
+    ecx.read_scalar_at_offset_atomic(mutex_op, 4, ecx.machine.layouts.u32, AtomicReadOp::Relaxed)
 }
 
 fn mutex_set_id<'mir, 'tcx: 'mir>(
@@ -96,8 +97,11 @@ fn mutex_set_id<'mir, 'tcx: 'mir>(
     id: impl Into<ScalarMaybeUninit<Tag>>,
 ) -> InterpResult<'tcx, ()> {
     ecx.write_scalar_at_offset_atomic(
-        mutex_op, 4, id, ecx.machine.layouts.u32,
-        AtomicWriteOp::Relaxed
+        mutex_op,
+        4,
+        id,
+        ecx.machine.layouts.u32,
+        AtomicWriteOp::Relaxed,
     )
 }
 
@@ -128,10 +132,7 @@ fn rwlock_get_id<'mir, 'tcx: 'mir>(
     ecx: &MiriEvalContext<'mir, 'tcx>,
     rwlock_op: &OpTy<'tcx, Tag>,
 ) -> InterpResult<'tcx, ScalarMaybeUninit<Tag>> {
-    ecx.read_scalar_at_offset_atomic(
-        rwlock_op, 4, ecx.machine.layouts.u32,
-        AtomicReadOp::Relaxed
-    )
+    ecx.read_scalar_at_offset_atomic(rwlock_op, 4, ecx.machine.layouts.u32, AtomicReadOp::Relaxed)
 }
 
 fn rwlock_set_id<'mir, 'tcx: 'mir>(
@@ -140,8 +141,11 @@ fn rwlock_set_id<'mir, 'tcx: 'mir>(
     id: impl Into<ScalarMaybeUninit<Tag>>,
 ) -> InterpResult<'tcx, ()> {
     ecx.write_scalar_at_offset_atomic(
-        rwlock_op, 4, id, ecx.machine.layouts.u32,
-        AtomicWriteOp::Relaxed
+        rwlock_op,
+        4,
+        id,
+        ecx.machine.layouts.u32,
+        AtomicWriteOp::Relaxed,
     )
 }
 
@@ -195,10 +199,7 @@ fn cond_get_id<'mir, 'tcx: 'mir>(
     ecx: &MiriEvalContext<'mir, 'tcx>,
     cond_op: &OpTy<'tcx, Tag>,
 ) -> InterpResult<'tcx, ScalarMaybeUninit<Tag>> {
-    ecx.read_scalar_at_offset_atomic(
-        cond_op, 4, ecx.machine.layouts.u32,
-        AtomicReadOp::Relaxed
-    )
+    ecx.read_scalar_at_offset_atomic(cond_op, 4, ecx.machine.layouts.u32, AtomicReadOp::Relaxed)
 }
 
 fn cond_set_id<'mir, 'tcx: 'mir>(
@@ -207,8 +208,11 @@ fn cond_set_id<'mir, 'tcx: 'mir>(
     id: impl Into<ScalarMaybeUninit<Tag>>,
 ) -> InterpResult<'tcx, ()> {
     ecx.write_scalar_at_offset_atomic(
-        cond_op, 4, id, ecx.machine.layouts.u32,
-        AtomicWriteOp::Relaxed
+        cond_op,
+        4,
+        id,
+        ecx.machine.layouts.u32,
+        AtomicWriteOp::Relaxed,
     )
 }
 
@@ -359,8 +363,8 @@ pub trait EvalContextExt<'mir, 'tcx: 'mir>: crate::MiriEvalContextExt<'mir, 'tcx
     ) -> InterpResult<'tcx, i32> {
         let this = self.eval_context_mut();
 
-        let attr = this.read_scalar(attr_op)?.check_init()?;
-        let kind = if this.is_null(attr)? {
+        let attr = this.read_pointer(attr_op)?;
+        let kind = if this.ptr_is_null(attr)? {
             this.eval_libc("PTHREAD_MUTEX_DEFAULT")?
         } else {
             mutexattr_get_kind(this, attr_op)?.check_init()?
@@ -652,8 +656,8 @@ pub trait EvalContextExt<'mir, 'tcx: 'mir>: crate::MiriEvalContextExt<'mir, 'tcx
     ) -> InterpResult<'tcx, i32> {
         let this = self.eval_context_mut();
 
-        let attr = this.read_scalar(attr_op)?.check_init()?;
-        let clock_id = if this.is_null(attr)? {
+        let attr = this.read_pointer(attr_op)?;
+        let clock_id = if this.ptr_is_null(attr)? {
             this.eval_libc("CLOCK_REALTIME")?
         } else {
             condattr_get_clock_id(this, attr_op)?.check_init()?
@@ -722,7 +726,7 @@ pub trait EvalContextExt<'mir, 'tcx: 'mir>: crate::MiriEvalContextExt<'mir, 'tcx
 
         // Extract the timeout.
         let clock_id = cond_get_clock_id(this, cond_op)?.to_i32()?;
-        let duration = match this.read_timespec(abstime_op)? {
+        let duration = match this.read_timespec(&this.deref_operand(abstime_op)?)? {
             Some(duration) => duration,
             None => {
                 let einval = this.eval_libc("EINVAL")?;
