@@ -65,32 +65,35 @@ pub(crate) fn inline_local_variable(acc: &mut Assists, ctx: &AssistContext) -> O
                         Some(u) => u,
                         None => return Some(false),
                     };
-
-                    Some(!matches!(
-                        (&initializer_expr, usage_parent),
-                        (ast::Expr::CallExpr(_), _)
-                            | (ast::Expr::IndexExpr(_), _)
-                            | (ast::Expr::MethodCallExpr(_), _)
-                            | (ast::Expr::FieldExpr(_), _)
-                            | (ast::Expr::TryExpr(_), _)
-                            | (ast::Expr::RefExpr(_), _)
-                            | (ast::Expr::Literal(_), _)
-                            | (ast::Expr::TupleExpr(_), _)
-                            | (ast::Expr::ArrayExpr(_), _)
-                            | (ast::Expr::ParenExpr(_), _)
-                            | (ast::Expr::PathExpr(_), _)
-                            | (ast::Expr::BlockExpr(_), _)
-                            | (ast::Expr::EffectExpr(_), _)
-                            | (_, ast::Expr::CallExpr(_))
-                            | (_, ast::Expr::TupleExpr(_))
-                            | (_, ast::Expr::ArrayExpr(_))
-                            | (_, ast::Expr::ParenExpr(_))
-                            | (_, ast::Expr::ForExpr(_))
-                            | (_, ast::Expr::WhileExpr(_))
-                            | (_, ast::Expr::BreakExpr(_))
-                            | (_, ast::Expr::ReturnExpr(_))
-                            | (_, ast::Expr::MatchExpr(_))
-                    ))
+                    let initializer = matches!(
+                        initializer_expr,
+                        ast::Expr::CallExpr(_)
+                            | ast::Expr::IndexExpr(_)
+                            | ast::Expr::MethodCallExpr(_)
+                            | ast::Expr::FieldExpr(_)
+                            | ast::Expr::TryExpr(_)
+                            | ast::Expr::RefExpr(_)
+                            | ast::Expr::Literal(_)
+                            | ast::Expr::TupleExpr(_)
+                            | ast::Expr::ArrayExpr(_)
+                            | ast::Expr::ParenExpr(_)
+                            | ast::Expr::PathExpr(_)
+                            | ast::Expr::BlockExpr(_)
+                            | ast::Expr::EffectExpr(_),
+                    );
+                    let parent = matches!(
+                        usage_parent,
+                        ast::Expr::CallExpr(_)
+                            | ast::Expr::TupleExpr(_)
+                            | ast::Expr::ArrayExpr(_)
+                            | ast::Expr::ParenExpr(_)
+                            | ast::Expr::ForExpr(_)
+                            | ast::Expr::WhileExpr(_)
+                            | ast::Expr::BreakExpr(_)
+                            | ast::Expr::ReturnExpr(_)
+                            | ast::Expr::MatchExpr(_)
+                    );
+                    Some(!(initializer || parent))
                 })
                 .collect::<Option<_>>()
                 .map(|b| (file_id, b))
