@@ -1,4 +1,19 @@
-//! FIXME: write short doc here
+//! In rust-analyzer, we maintain a strict separation between pure abstract
+//! semantic project model and a concrete model of a particular build system.
+//!
+//! Pure model is represented by the [`base_db::CrateGraph`] from another crate.
+//!
+//! In this crate, we are conserned with "real world" project models.
+//!
+//! Specifically, here we have a representation for a Cargo project
+//! ([`CargoWorkspace`]) and for manually specified layout ([`ProjectJson`]).
+//!
+//! Roughly, the things we do here are:
+//!
+//! * Project discovery (where's the relevant Cargo.toml for the current dir).
+//! * Custom build steps (`build.rs` code generation and compilation of
+//!   procedural macros).
+//! * Lowering of concrete model to a [`base_db::CrateGraph`]
 
 mod cargo_workspace;
 mod cfg_flag;
@@ -9,7 +24,7 @@ mod rustc_cfg;
 mod build_data;
 
 use std::{
-    fs::{read_dir, ReadDir},
+    fs::{self, read_dir, ReadDir},
     io,
     process::Command,
 };
@@ -21,12 +36,12 @@ use rustc_hash::FxHashSet;
 pub use crate::{
     build_data::{BuildDataCollector, BuildDataResult},
     cargo_workspace::{
-        CargoConfig, CargoWorkspace, Package, PackageData, PackageDependency, Target, TargetData,
-        TargetKind,
+        CargoConfig, CargoWorkspace, Package, PackageData, PackageDependency, RustcSource, Target,
+        TargetData, TargetKind,
     },
     project_json::{ProjectJson, ProjectJsonData},
     sysroot::Sysroot,
-    workspace::{PackageRoot, ProjectWorkspace},
+    workspace::{CfgOverrides, PackageRoot, ProjectWorkspace},
 };
 
 pub use proc_macro_api::ProcMacroClient;
@@ -39,10 +54,10 @@ pub enum ProjectManifest {
 
 impl ProjectManifest {
     pub fn from_manifest_file(path: AbsPathBuf) -> Result<ProjectManifest> {
-        if path.ends_with("rust-project.json") {
+        if path.file_name().unwrap_or_default() == "rust-project.json" {
             return Ok(ProjectManifest::ProjectJson(path));
         }
-        if path.ends_with("Cargo.toml") {
+        if path.file_name().unwrap_or_default() == "Cargo.toml" {
             return Ok(ProjectManifest::CargoToml(path));
         }
         bail!("project root must point to Cargo.toml or rust-project.json: {}", path.display())
@@ -76,7 +91,7 @@ impl ProjectManifest {
         }
 
         fn find_in_parent_dirs(path: &AbsPath, target_file_name: &str) -> Option<AbsPathBuf> {
-            if path.ends_with(target_file_name) {
+            if path.file_name().unwrap_or_default() == target_file_name {
                 return Some(path.to_path_buf());
             }
 
@@ -84,7 +99,7 @@ impl ProjectManifest {
 
             while let Some(path) = curr {
                 let candidate = path.join(target_file_name);
-                if candidate.exists() {
+                if fs::metadata(&candidate).is_ok() {
                     return Some(candidate);
                 }
                 curr = path.parent();
@@ -104,7 +119,7 @@ impl ProjectManifest {
         }
     }
 
-    pub fn discover_all(paths: &[impl AsRef<AbsPath>]) -> Vec<ProjectManifest> {
+    pub fn discover_all(paths: &[AbsPathBuf]) -> Vec<ProjectManifest> {
         let mut res = paths
             .iter()
             .filter_map(|it| ProjectManifest::discover(it.as_ref()).ok())
